@@ -1,7 +1,7 @@
 /*
  * <security/_pam_types.h>
  *
- * $Id: _pam_types.h,v 1.10 1997/04/05 06:52:50 morgan Exp morgan $
+ * $Id: _pam_types.h,v 1.4 1999/02/22 02:29:18 morgan Exp $
  *
  * This file defines all of the types common to the Linux-PAM library
  * applications and modules.
@@ -10,7 +10,19 @@
  *
  * Created: 1996/3/5 by AGM
  *
- * $Log$
+ * $Log: _pam_types.h,v $
+ * Revision 1.4  1999/02/22 02:29:18  morgan
+ * fix(?) for strange glibc behavior wrt LOG_PRIV
+ *
+ * Revision 1.3  1998/12/27 04:06:48  morgan
+ * retracted syslog functions.. PAM should not be trying to solve libc problems
+ *
+ * Revision 1.2  1998/07/12 05:57:18  morgan
+ * update for systems not having LOG_AUTHPRIV syslog level
+ *
+ * Revision 1.1.1.1  1998/07/12 05:17:15  morgan
+ * Linux PAM sources pre-0.66
+ *
  */
 
 #ifndef _SECURITY__PAM_TYPES_H
@@ -141,7 +153,6 @@ typedef struct pam_handle pam_handle_t;
 #define PAM_USER_PROMPT    9    /* the prompt for getting a username */
 #define PAM_FAIL_DELAY     10   /* app supplied function to override failure
 				   delays */
-#define PAM_LOG_STATE      11   /* ident, facility etc. logging info */
 
 /* ---------- Common Linux-PAM application/module PI ----------- */
 
@@ -184,44 +195,12 @@ extern char **pam_getenvlist(pam_handle_t *pamh);
 #define HAVE_PAM_FAIL_DELAY
 extern int pam_fail_delay(pam_handle_t *pamh, unsigned int musec_delay);
 
-/*
- * the standard libc interface for syslog suffers from some problems.
- * The first is that it is not thread safe.  It is also three functions
- * where PAM only really needs a "log this" function.  It also does
- * not provide modules and applications with information about whether
- * the log is currently open or not etc...  All of these things mean
- * that we need to centralize PAM's logging facility.  These two functions
- * provide this centralization.  They are, however, just a gateway to
- * libc's openlog/syslog/closelog functions.  Please note, your apps/modules
- * will likely start to segfault if you do not use this function for
- * system logging.
- */
-
-struct pam_log_state {
-    char *ident;
-    int option;
-    int facility;
-};
-
-#ifndef LOG_ERR
-# include <syslog.h>      /* this is a sad HACK. But we need LOG_CRIT etc.. */
-#endif
-
-#define PAM_LOG_STATE_IDENT    "PAM"
-#define PAM_LOG_STATE_OPTION   LOG_PID
-#define PAM_LOG_STATE_FACILITY LOG_AUTHPRIV
-
-#ifndef va_start
-# include <stdarg.h>
-#endif
-
-#define HAVE_PAM_SYSTEM_LOG
-extern void pam_vsystem_log(const pam_handle_t *pamh,
-			    const struct pam_log_state *log_state,
-			    int priority, const char *format, va_list args);
-extern void pam_system_log(const pam_handle_t *pamh,
-			   const struct pam_log_state *log_state, 
-			   int priority, const char *format, ... );
+#include <syslog.h>
+#ifndef LOG_AUTHPRIV
+# ifdef LOG_PRIV
+#  define LOG_AUTHPRIV LOG_PRIV
+# endif /* LOG_PRIV */
+#endif /* !LOG_AUTHPRIV */
 
 #ifdef MEMORY_DEBUG
 /*
