@@ -1,33 +1,20 @@
 /*
  * pam_delay.c
  *
- * Copyright (c) Andrew G. Morgan <morgan@parc.power.net> 1996
+ * Copyright (c) Andrew G. Morgan <morgan@linux.kernel.org> 1996-8
  * All rights reserved.
  *
- * $Id: pam_delay.c,v 1.5 1997/04/05 06:54:19 morgan Exp $
+ * $Id: pam_delay.c,v 1.1.1.1 1998/07/12 05:17:15 morgan Exp $
  *
  * $Log: pam_delay.c,v $
- * Revision 1.5  1997/04/05 06:54:19  morgan
- * conditional compilation removed
- *
- * Revision 1.4  1996/12/01 03:14:13  morgan
- * use _pam_macros.h
- *
- * Revision 1.3  1996/11/10 20:00:57  morgan
- * POSIX delays
- *
- * Revision 1.2  1996/09/05 06:11:50  morgan
- * changed delays to be pseudo gausian.
- *
- * Revision 1.1  1996/07/08 00:07:07  morgan
- * Initial revision
+ * Revision 1.1.1.1  1998/07/12 05:17:15  morgan
+ * Linux PAM sources pre-0.66
  *
  */
 
 /*
- * this file is likely to change/disappear. Currently it is a simple
- * implementation of a delay on failure mechanism; an attempt to
- * overcome authentication-time attacts in a simple manner.
+ * This is a simple implementation of a delay on failure mechanism; an
+ * attempt to overcome authentication-time attacks in a simple manner.
  */
 
 #include <unistd.h>
@@ -102,26 +89,37 @@ static unsigned int _pam_compute_delay(unsigned int seed, unsigned int base)
 
 void _pam_await_timer(pam_handle_t *pamh, int status)
 {
-     D(("waiting?..."));
+    unsigned int delay;
+    D(("waiting?..."));
 
-     if (status != PAM_SUCCESS && pamh->fail_delay.set) {
-          int delay;
+    delay = _pam_compute_delay(pamh->fail_delay.begin,
+			       pamh->fail_delay.delay);
+    if (pamh->fail_delay.delay_fn_ptr) {
+	union {
+	    const void *value;
+	    void (*fn)(int, unsigned);
+	} hack_fn_u;
 
-	  delay = _pam_compute_delay(pamh->fail_delay.begin
-				     , pamh->fail_delay.delay);
-	  D(("will wait %u usec", delay));
+	/* always call the applications delay function, even if
+	   the delay is zero - indicate status */
+	hack_fn_u.value = pamh->fail_delay.delay_fn_ptr;
+	hack_fn_u.fn(status, delay);
 
-	  if (delay > 0) {
-	       struct timeval tval;
+    } else if (status != PAM_SUCCESS && pamh->fail_delay.set) {
 
-	       tval.tv_sec  = delay / 1000000;
-	       tval.tv_usec = delay % 1000000;
-	       select(0, NULL, NULL, NULL, &tval);
-	  }
-     }
+	D(("will wait %u usec", delay));
 
-     _pam_reset_timer(pamh);
-     D(("waiting done"));
+	if (delay > 0) {
+	    struct timeval tval;
+
+	    tval.tv_sec  = delay / 1000000;
+	    tval.tv_usec = delay % 1000000;
+	    select(0, NULL, NULL, NULL, &tval);
+	}
+    }
+
+    _pam_reset_timer(pamh);
+    D(("waiting done"));
 }
 
 /* **********************************************************************
@@ -134,7 +132,7 @@ int pam_fail_delay(pam_handle_t *pamh, unsigned int usec)
 {
      int largest;
 
-     IF_NO_PAMH("pam_fail_delay",pamh,PAM_SYSTEM_ERR);
+     IF_NO_PAMH("pam_fail_delay", pamh, PAM_SYSTEM_ERR);
 
      D(("setting delay to %u",usec));
 
