@@ -1,4 +1,4 @@
-/* $Header: /home/morgan/pam/Linux-PAM-0.59/modules/pam_unix/RCS/pam_unix_auth.c,v 1.1 1996/11/09 19:44:35 morgan Exp morgan $ */
+/* $Id: pam_unix_auth.c,v 1.2 1999/06/15 05:45:14 morgan Exp $ */
 
 /*
  * Copyright Alexander O. Yuriev, 1996.  All rights reserved.
@@ -36,42 +36,6 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
  
-/* 
- * $Log: pam_unix_auth.c,v $
- *
- * Revision 1.9  1996/05/26 04:13:04  morgan
- * added static support
- *
- * Revision 1.8  1996/05/21 03:51:58  morgan
- * added "const" to rcsid[] definition
- *
- * Revision 1.7  1996/04/19 03:25:57  alex
- * minor corrections.
- *
- * Revision 1.6  1996/04/17 01:05:05  alex
- * _pam_auth_unix() cleaned up - non-authentication code is made into funcs
- * and mostly moved out to support.c.
- *
- * Revision 1.5  1996/04/16 21:12:46  alex
- * unix authentication works on Bach again. This is a tranitional stage.
- * I really don't like that _pam_unix_auth() grew into a monster that does
- * prompts etc etc. They should go into other functions.
- *
- * Revision 1.4  1996/04/07 08:06:12  morgan
- * tidied up a little
- *
- * Revision 1.3  1996/04/07 07:34:07  morgan
- * added conversation support. Now the module is capable of obtaining a
- * username and a password all by itself.
- *
- * Revision 1.2  1996/03/29 02:31:19  morgan
- * Marek Michalkiewicz's small patches for shadow support.
- *
- * Revision 1.1  1996/03/09 09:10:57  morgan
- * Initial revision
- *
- */
-
 #ifdef linux
 # define _GNU_SOURCE
 # include <features.h>
@@ -85,18 +49,16 @@
 #include <pwd.h>
 #include <sys/types.h>
 
-#ifndef NDEBUG
-
 #include <syslog.h>
 
-#endif	/* NDEBUG */
-
 #ifdef HAVE_SHADOW_H
 
 #include <shadow.h>
 	
 #endif	/* HAVE_SHADOW_H */
 
+#include <security/_pam_macros.h>
+
 #ifndef LINUX 
 
 #include <security/pam_appl.h>
@@ -106,9 +68,20 @@
 #define _PAM_EXTERN_FUNCTIONS
 #include <security/pam_modules.h>
 
-static const char rcsid[] = "$Id: pam_unix_auth.c,v 1.1 1996/11/09 19:44:35 morgan Exp morgan $ pam_unix authentication functions. alex@bach.cis.temple.edu";
+/* defines for syslog verbosity */
+#define VERBOSE_DEBUG 1
+#define VERBOSE_AUDIT 2
 
-/* Define function phototypes */
+/* How shall we get the password from the user? */
+#define PASS_PROMPT_USER 0
+#define PASS_TRY_FIRST 1
+#define PASS_USE_FIRST 2
+
+static const char rcsid[] = "$Id: pam_unix_auth.c,v 1.2 1999/06/15 05:45:14 morgan Exp $ pam_unix authentication functions. alex@bach.cis.temple.edu";
+
+static int verbose = 0;
+
+/* Define function prototypes */
 
 extern char *crypt(const char *key, const char *salt);	/* This should have 
 							   been in unistd.h
@@ -188,16 +161,6 @@ static int _pam_auth_unix(	pam_handle_t *pamh,
 
 	pw = getpwnam ( name );
 
-	/* For NIS+, root cannot get password for lesser user */
-	if (pw) {
-	    uid_t save_uid;
-
-	    save_uid = geteuid ();
-	    if (seteuid (pw->pw_uid) >= 0) {
-		pw = getpwnam ( name );
-		seteuid (save_uid);
-	    }
-	}
 	if (pw) 
 		{
 
@@ -209,8 +172,16 @@ static int _pam_auth_unix(	pam_handle_t *pamh,
 		 * there is no shadow password, use the non-shadow one.
 		 */
 
-		sp = getspnam( name );
-		if (sp && (!strcmp(pw->pw_passwd,"x")))
+		if (!strcmp(pw->pw_passwd,"*NP*")) { /* NIS+ */
+		    uid_t save_uid;
+		    save_uid = getuid();
+		    seteuid (pw->pw_uid);
+		    sp = getspnam( name );
+		    seteuid (save_uid);
+		} else
+		    sp = getspnam( name );
+		
+		if (sp && (strlen(pw->pw_passwd) < 13))
 			{
 				/* TODO: check if password has expired etc. */
 				salt = sp->sp_pwdp;
@@ -232,9 +203,9 @@ static int _pam_auth_unix(	pam_handle_t *pamh,
 		else
 			return PAM_AUTH_ERR;
 				
-	pp = crypt(p, salt);
+	pp = p ? crypt(p, salt) : NULL;
 	
-	if ( strcmp( pp, salt ) == 0 ) 
+	if ( pp && strcmp( pp, salt ) == 0 ) 
 		return	PAM_SUCCESS;
 
   	return PAM_AUTH_ERR;
