@@ -57,6 +57,7 @@
 #define _BSD_SOURCE
 #include <pwd.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 
 /* why not defined? */
 void setpwent(void);
@@ -87,6 +88,8 @@ char *crypt(const char *key, const char *salt);
 #include <shadow.h>
 #endif
 
+#include "md5.h"
+
 #define MAX_PASSWD_TRIES 3
 #define OLD_PASSWORD_PROMPT "Password: "
 #define NEW_PASSWORD_PROMPT "New password: "
@@ -104,6 +107,11 @@ char *crypt(const char *key, const char *salt);
 #define PPW_TOOEARLY 32
 #define PPW_ERROR 64
 
+/* Password hash types available. For now, this includes only standard
+   crypt and md5, but the list may be expanded as needed. */
+#define HASH_CRYPT 0
+#define HASH_MD5 1
+
 #ifndef DO_TEST
 #define STATIC static
 #else
@@ -111,7 +119,7 @@ char *crypt(const char *key, const char *salt);
 #endif
 /* Sets a password for the specified user to the specified password
    Returns flags PPW_*, or'd. */
-STATIC int _do_setpass(char *forwho, char *towhat, int flags);
+STATIC int _do_setpass(char *forwho, char *towhat, int crypt_type, int flags);
 /* Gets a password for the specified user
    Returns flags PPW_*, or'd. */
 STATIC int _do_getpass(char *forwho, char **theirpass);
@@ -119,7 +127,7 @@ STATIC int _do_getpass(char *forwho, char **theirpass);
    'entered' should not be crypt()'d or anything (it should be as the
    user entered it...), 'listed' should be as it is listed in the
    password database file */
-STATIC int _do_checkpass(const char *entered, char *listed);
+STATIC int _do_checkpass(const char *entered, const char *listed);
 
 /* sends a one-way message to the user, either error or info... */
 STATIC int conv_sendmsg(struct pam_conv *aconv, const char *message, int style);
@@ -127,6 +135,10 @@ STATIC int conv_sendmsg(struct pam_conv *aconv, const char *message, int style);
 STATIC int conv_getitem(struct pam_conv *aconv, char *message, int style,
 			  char **result);
 
+/* mangles an integer into the password file's special printable character
+   format. */
+STATIC char i64c(int i);
+
 PAM_EXTERN
 int pam_sm_chauthtok(	pam_handle_t *pamh, 
 			int flags,
@@ -291,6 +303,9 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
   int fascist = 1; /* Be fascist by default.  If compiled with cracklib,
                       call cracklib.  Otherwise just check length... */
 
+  int crypt_type = HASH_CRYPT; /* What type of salt do we use for
+                                  encrypting? Standard unix crypt is the
+                                  default. */
   char argbuf[256],argval[256];
   int i;
 
@@ -327,15 +342,23 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
      }
 
      /* For PC functionality use "strict" -- historically "fascist" */
-     if(!strcmp(argbuf,"strict") || !strcmp(argbuf, "fascist"))
-
+     if(!strcmp(argbuf,"strict") || !strcmp(argbuf, "fascist")) {
        if(!strcmp(argval,"true"))
          fascist = 1;
        else if(!strcmp(argval,"false"))
          fascist = 0;
        else
          return PAM_SERVICE_ERR;
-     else {
+     } else if(!strcmp(argbuf,"hash")) {
+       if(!strcmp(argval,"crypt"))     /* Default. */
+         crypt_type = HASH_CRYPT;
+#ifdef HAVE_CRYPT_MD5
+       else if(!strcmp(argval,"md5"))
+         crypt_type = HASH_MD5;
+#endif
+       else
+          return PAM_SERVICE_ERR;
+     } else {
        _pam_log(LOG_ERR,"Unknown option: %s",argbuf);
        return PAM_SERVICE_ERR;
      }
@@ -371,7 +394,7 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
       PAM_FAIL_CHECK;
       curpass = resp->resp;
       free (resp);
-      if(_do_checkpass(curpass?curpass:"",miscptr)) {
+      if(_do_checkpass(curpass?curpass:"",(const char *)miscptr)) {
         int abortme = 0;
 
         /* password is incorrect... */
@@ -443,7 +466,7 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
     if(newpass) {
 #ifdef USE_CRACKLIB
       if(fascist && getuid()) 
-        cmiscptr = FascistCheck(newpass,CRACKLIB_DICTS);
+        cmiscptr = FascistCheck(newpass, CRACKLIB_DICTS);
 #else
       if(fascist && getuid() && strlen(newpass) < 6)
 	cmiscptr = "You must choose a longer password";
@@ -504,7 +527,7 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
   /* From now on, we are bound and determined to get their password
      changed :-) */
   pam_set_item(pamh, PAM_AUTHTOK, (void *)newpass);
-  retval = _do_setpass(usrname,newpass,pflags);
+  retval = _do_setpass(usrname,newpass,crypt_type,pflags);
 #ifdef DEBUG
     fprintf(stderr,"retval was %d\n",retval);
 #endif
@@ -518,50 +541,83 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
 }
 
 /* _do_checkpass() returns 0 on success, non-0 on failure */
-STATIC int _do_checkpass(const char *entered, char *listed)
+STATIC int _do_checkpass(const char *entered, const char *listed)
 {
-  char salt[3];
   if ((strlen(listed) == 0) &&(strlen(entered) == 0)) {
     /* no password in database; no password entered */
     return (0);
   }
-  salt[0]=listed[0]; salt[1]=listed[1]; salt[2]='\0';
-  return strcmp(crypt(entered,salt),listed);
+
+/* This guarantees that the salt is not inappropriately truncated when
+   dealing with alternative hashing algorithms, but are there possibly
+   crypt() implementations that *fail* if not passed a two-byte salt?  Not
+   a problem on Linux, but we should try to be cross-platform. -SRL */
+
+  return entered ? strcmp(crypt(entered,listed),listed) : -1;
 }
 
-STATIC char mksalt(int seed) {
-  int num = seed % 64;
-
-  if (num < 26)
-    return 'a' + num;
-  else if (num < 52)
-    return 'A' + (num - 26);
-  else if (num < 62)
-    return '0' + (num - 52);
-  else if (num == 63)
-    return '.';
-  else
-    return '/';
+STATIC char i64c(int i) {
+  if (i < 0)
+    return ('.');
+  else if (i > 63)
+    return ('z');
+  if (i == 0)
+    return ('.');
+  if (i == 1)
+    return ('/');
+  if (i >= 2 && i <= 11)
+    return ('0' - 2 + i);
+  if (i >= 12 && i <= 37)
+    return ('A' - 12 + i);
+  if (i >= 38 && i <= 63)
+    return ('a' - 38 + i);
+  return ('\0');
 }
 
-STATIC int _do_setpass(char *forwho, char *towhat,int flags)
+STATIC int _do_setpass(char *forwho, char *towhat,int crypt_type, int flags)
 {
   struct passwd *pwd=NULL, *tmpent=NULL;
   FILE *pwfile,*opwfile;
-  char thesalt[3];
+  struct timeval tv;
+  MD5_CTX ctx;
+  unsigned char tmp[16];
+  unsigned char thesalt[16];
+  char *cp = (char *)thesalt;
+  int i;
   int retval=0;
-  struct timeval time1;
   int err = 0;
 #ifdef HAVE_SHADOW_H
   struct spwd *spwdent=NULL, *stmpent=NULL;
+  int oldmask;
 #endif
+
   if(flags & PPW_SHADOW) { retval |= PPW_SHADOW; }
-  gettimeofday(&time1, NULL);
-  srand(time1.tv_usec);
-  thesalt[0]=mksalt(rand());
-  thesalt[1]=mksalt(rand());
-  thesalt[2]='\0';
-  
+
+  MD5Init(&ctx);
+  gettimeofday(&tv, (struct timezone *) 0);
+  MD5Update(&ctx, (void *) &tv, sizeof tv);
+  i = getpid();
+  MD5Update(&ctx, (void *) &i, sizeof i);
+  i = clock();
+  MD5Update(&ctx, (void *) &i, sizeof i);
+  MD5Update(&ctx, thesalt, sizeof thesalt);
+  MD5Final(tmp, &ctx);
+
+  if(crypt_type == HASH_CRYPT) {
+    thesalt[0]=i64c(tmp[0] & 077);
+    thesalt[1]=i64c(tmp[1] & 077);
+    thesalt[2]='\0';
+  }
+#ifdef HAVE_CRYPT_MD5
+  else if(crypt_type == HASH_MD5) {
+    strcpy(cp, "$1$");  /* magic for the MD5 */
+    cp += strlen(cp);
+    for (i = 0; i < 8; i++)
+      *cp++ = i64c(tmp[i] & 077);
+    *cp = '\0';
+  }
+#endif
+
   /* lock the entire password subsystem */
 #ifdef USE_LCKPWDF
   lckpwdf();
@@ -583,7 +639,9 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
       return PPW_NOSUCHUSER;
     spwdent->sp_pwdp = towhat;
     spwdent->sp_lstchg = time(NULL)/(60*60*24);
+    oldmask = umask(077);
     pwfile = fopen(SH_TMPFILE,"w");
+    umask(oldmask);
     opwfile = fopen("/etc/shadow","r");
     if(pwfile == NULL || opwfile == NULL)
       return PPW_ERROR;
@@ -592,7 +650,7 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
     stmpent=fgetspent(opwfile);
     while(stmpent) {
       if(!strcmp(stmpent->sp_namp,forwho)) {
-	stmpent->sp_pwdp = crypt(towhat,thesalt);
+	stmpent->sp_pwdp = towhat ? crypt(towhat,thesalt) : NULL;
 	stmpent->sp_lstchg = time(NULL)/(60*60*24);
 #ifdef DEBUG
 	fprintf(stderr,"Set password %s for %s\n",stmpent->sp_pwdp,
@@ -623,7 +681,9 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
       unlink(SH_TMPFILE);
   } else {
     pwd->pw_passwd = towhat;
+    oldmask = umask(077);
     pwfile = fopen(PW_TMPFILE,"w");
+    umask(oldmask);
     opwfile = fopen("/etc/passwd","r");
     if(pwfile == NULL || opwfile == NULL)
       return PPW_ERROR;
@@ -632,7 +692,7 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
     tmpent=fgetpwent(opwfile);
     while(tmpent) {
       if(!strcmp(tmpent->pw_name,forwho)) {
-	tmpent->pw_passwd = crypt(towhat,thesalt);
+	tmpent->pw_passwd = towhat ? crypt(towhat,thesalt) : NULL;
       }
       if (putpwent(tmpent,pwfile)) {
 	fprintf(stderr, "error writing entry to password file: %s\n",
@@ -659,7 +719,9 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
   }
 #else
   pwd->pw_passwd = towhat;
+  oldmask = umask(077);
   pwfile = fopen(PW_TMPFILE,"w");
+  umask(oldmask);
   opwfile = fopen("/etc/passwd","r");
   if(pwfile == NULL || opwfile == NULL)
     return PPW_ERROR;
@@ -668,7 +730,7 @@ STATIC int _do_setpass(char *forwho, char *towhat,int flags)
   tmpent=fgetpwent(opwfile);
   while(tmpent) {
     if(!strcmp(tmpent->pw_name,forwho)) {
-      tmpent->pw_passwd = crypt(towhat,thesalt);
+      tmpent->pw_passwd = towhat ? crypt(towhat,thesalt) : NULL;
     }
     if (putpwent(tmpent,pwfile)) {
 	fprintf(stderr, "error writing entry to shadow file: %s\n",
