@@ -171,10 +171,12 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
          }
      }
      
-     if (!use_group[0])
-         grp = getgrgid(0);
-     else
-        grp = getgrnam(use_group);
+     if (!use_group[0]) {
+	 if ((grp = getgrnam("wheel")) == NULL) {
+	     grp = getgrgid(0);
+	 }
+     } else
+	 grp = getgrnam(use_group);
         
      if (!grp || !grp->gr_mem) {
         if (ctrl & PAM_DEBUG_ARG) {
